// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type MaxWorkLocationDetails int

const (
    UNKNOWN_MAXWORKLOCATIONDETAILS MaxWorkLocationDetails = iota
    NONE_MAXWORKLOCATIONDETAILS
    APPROXIMATE_MAXWORKLOCATIONDETAILS
    SPECIFIC_MAXWORKLOCATIONDETAILS
    UNKNOWNFUTUREVALUE_MAXWORKLOCATIONDETAILS
)

func (i MaxWorkLocationDetails) String() string {
    return []string{"unknown", "none", "approximate", "specific", "unknownFutureValue"}[i]
}
func ParseMaxWorkLocationDetails(v string) (any, error) {
    result := UNKNOWN_MAXWORKLOCATIONDETAILS
    switch v {
        case "unknown":
            result = UNKNOWN_MAXWORKLOCATIONDETAILS
        case "none":
            result = NONE_MAXWORKLOCATIONDETAILS
        case "approximate":
            result = APPROXIMATE_MAXWORKLOCATIONDETAILS
        case "specific":
            result = SPECIFIC_MAXWORKLOCATIONDETAILS
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_MAXWORKLOCATIONDETAILS
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeMaxWorkLocationDetails(values []MaxWorkLocationDetails) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i MaxWorkLocationDetails) isMultiValue() bool {
    return false
}
