// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package identitygovernance

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
)

// Deprecated: This class is obsolete. Use AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponseable instead.
type AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse struct {
    AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponse
}
// NewAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse instantiates a new AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse and sets the default values.
func NewAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse()(*AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse) {
    m := &AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse{
        AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponse: *NewAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponse(),
    }
    return m
}
// CreateAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponseFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponseFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewAccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponse(), nil
}
// Deprecated: This class is obsolete. Use AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponseable instead.
type AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnResponseable interface {
    AccessReviewsDefinitionsItemInstancesItemStagesFilterByCurrentUserWithOnGetResponseable
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
}
