// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"
	"encoding/json"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/validate"
)

// HashicorpCloudIamUserPrincipalIdentityType IdentityType contains the identity types for users.
//
//   - UNSET: UNSET is the default value, should not be used.
//   - SOCIAL_GITHUB: SOCIAL_GITHUB is the GitHub Social login provider.
//   - EMAIL_PASSWORD: EMAIL_PASSWORD is the auth0 email/password database provider.
//   - SAMLP: SAMLP is the auth0 SAML provider.
//   - HASHICORP_SSO: HASHICORP_SSO is the HashiCorp SSO provider.
//
// swagger:model hashicorp.cloud.iam.UserPrincipal.IdentityType
type HashicorpCloudIamUserPrincipalIdentityType string

func NewHashicorpCloudIamUserPrincipalIdentityType(value HashicorpCloudIamUserPrincipalIdentityType) *HashicorpCloudIamUserPrincipalIdentityType {
	return &value
}

// Pointer returns a pointer to a freshly-allocated HashicorpCloudIamUserPrincipalIdentityType.
func (m HashicorpCloudIamUserPrincipalIdentityType) Pointer() *HashicorpCloudIamUserPrincipalIdentityType {
	return &m
}

const (

	// HashicorpCloudIamUserPrincipalIdentityTypeUNSET captures enum value "UNSET"
	HashicorpCloudIamUserPrincipalIdentityTypeUNSET HashicorpCloudIamUserPrincipalIdentityType = "UNSET"

	// HashicorpCloudIamUserPrincipalIdentityTypeSOCIALGITHUB captures enum value "SOCIAL_GITHUB"
	HashicorpCloudIamUserPrincipalIdentityTypeSOCIALGITHUB HashicorpCloudIamUserPrincipalIdentityType = "SOCIAL_GITHUB"

	// HashicorpCloudIamUserPrincipalIdentityTypeEMAILPASSWORD captures enum value "EMAIL_PASSWORD"
	HashicorpCloudIamUserPrincipalIdentityTypeEMAILPASSWORD HashicorpCloudIamUserPrincipalIdentityType = "EMAIL_PASSWORD"

	// HashicorpCloudIamUserPrincipalIdentityTypeSAMLP captures enum value "SAMLP"
	HashicorpCloudIamUserPrincipalIdentityTypeSAMLP HashicorpCloudIamUserPrincipalIdentityType = "SAMLP"

	// HashicorpCloudIamUserPrincipalIdentityTypeHASHICORPSSO captures enum value "HASHICORP_SSO"
	HashicorpCloudIamUserPrincipalIdentityTypeHASHICORPSSO HashicorpCloudIamUserPrincipalIdentityType = "HASHICORP_SSO"
)

// for schema
var hashicorpCloudIamUserPrincipalIdentityTypeEnum []interface{}

func init() {
	var res []HashicorpCloudIamUserPrincipalIdentityType
	if err := json.Unmarshal([]byte(`["UNSET","SOCIAL_GITHUB","EMAIL_PASSWORD","SAMLP","HASHICORP_SSO"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		hashicorpCloudIamUserPrincipalIdentityTypeEnum = append(hashicorpCloudIamUserPrincipalIdentityTypeEnum, v)
	}
}

func (m HashicorpCloudIamUserPrincipalIdentityType) validateHashicorpCloudIamUserPrincipalIdentityTypeEnum(path, location string, value HashicorpCloudIamUserPrincipalIdentityType) error {
	if err := validate.EnumCase(path, location, value, hashicorpCloudIamUserPrincipalIdentityTypeEnum, true); err != nil {
		return err
	}
	return nil
}

// Validate validates this hashicorp cloud iam user principal identity type
func (m HashicorpCloudIamUserPrincipalIdentityType) Validate(formats strfmt.Registry) error {
	var res []error

	// value enum
	if err := m.validateHashicorpCloudIamUserPrincipalIdentityTypeEnum("", "body", m); err != nil {
		return err
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

// ContextValidate validates this hashicorp cloud iam user principal identity type based on context it is used
func (m HashicorpCloudIamUserPrincipalIdentityType) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}
