// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package teamsadministration
type NumberType int

const (
    INTERNALERROR_NUMBERTYPE NumberType = iota
    DIRECTROUTING_NUMBERTYPE
    CALLINGPLAN_NUMBERTYPE
    OPERATORCONNECT_NUMBERTYPE
    UNKNOWNFUTUREVALUE_NUMBERTYPE
)

func (i NumberType) String() string {
    return []string{"internalError", "directRouting", "callingPlan", "operatorConnect", "unknownFutureValue"}[i]
}
func ParseNumberType(v string) (any, error) {
    result := INTERNALERROR_NUMBERTYPE
    switch v {
        case "internalError":
            result = INTERNALERROR_NUMBERTYPE
        case "directRouting":
            result = DIRECTROUTING_NUMBERTYPE
        case "callingPlan":
            result = CALLINGPLAN_NUMBERTYPE
        case "operatorConnect":
            result = OPERATORCONNECT_NUMBERTYPE
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_NUMBERTYPE
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeNumberType(values []NumberType) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i NumberType) isMultiValue() bool {
    return false
}
