// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.6
// 	protoc        v5.29.3
// source: couchbase/query/v1/query.proto

package query_v1

import (
	kv_v1 "github.com/couchbase/goprotostellar/genproto/kv_v1"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	durationpb "google.golang.org/protobuf/types/known/durationpb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type QueryRequest_ScanConsistency int32

const (
	QueryRequest_SCAN_CONSISTENCY_NOT_BOUNDED  QueryRequest_ScanConsistency = 0
	QueryRequest_SCAN_CONSISTENCY_REQUEST_PLUS QueryRequest_ScanConsistency = 1
)

// Enum value maps for QueryRequest_ScanConsistency.
var (
	QueryRequest_ScanConsistency_name = map[int32]string{
		0: "SCAN_CONSISTENCY_NOT_BOUNDED",
		1: "SCAN_CONSISTENCY_REQUEST_PLUS",
	}
	QueryRequest_ScanConsistency_value = map[string]int32{
		"SCAN_CONSISTENCY_NOT_BOUNDED":  0,
		"SCAN_CONSISTENCY_REQUEST_PLUS": 1,
	}
)

func (x QueryRequest_ScanConsistency) Enum() *QueryRequest_ScanConsistency {
	p := new(QueryRequest_ScanConsistency)
	*p = x
	return p
}

func (x QueryRequest_ScanConsistency) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (QueryRequest_ScanConsistency) Descriptor() protoreflect.EnumDescriptor {
	return file_couchbase_query_v1_query_proto_enumTypes[0].Descriptor()
}

func (QueryRequest_ScanConsistency) Type() protoreflect.EnumType {
	return &file_couchbase_query_v1_query_proto_enumTypes[0]
}

func (x QueryRequest_ScanConsistency) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use QueryRequest_ScanConsistency.Descriptor instead.
func (QueryRequest_ScanConsistency) EnumDescriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{0, 0}
}

type QueryRequest_DurabilityLevel int32

const (
	QueryRequest_DURABILITY_LEVEL_NONE                           QueryRequest_DurabilityLevel = 0
	QueryRequest_DURABILITY_LEVEL_MAJORITY                       QueryRequest_DurabilityLevel = 1
	QueryRequest_DURABILITY_LEVEL_MAJORITY_AND_PERSIST_TO_ACTIVE QueryRequest_DurabilityLevel = 2
	QueryRequest_DURABILITY_LEVEL_PERSIST_TO_MAJORITY            QueryRequest_DurabilityLevel = 3
)

// Enum value maps for QueryRequest_DurabilityLevel.
var (
	QueryRequest_DurabilityLevel_name = map[int32]string{
		0: "DURABILITY_LEVEL_NONE",
		1: "DURABILITY_LEVEL_MAJORITY",
		2: "DURABILITY_LEVEL_MAJORITY_AND_PERSIST_TO_ACTIVE",
		3: "DURABILITY_LEVEL_PERSIST_TO_MAJORITY",
	}
	QueryRequest_DurabilityLevel_value = map[string]int32{
		"DURABILITY_LEVEL_NONE":                           0,
		"DURABILITY_LEVEL_MAJORITY":                       1,
		"DURABILITY_LEVEL_MAJORITY_AND_PERSIST_TO_ACTIVE": 2,
		"DURABILITY_LEVEL_PERSIST_TO_MAJORITY":            3,
	}
)

func (x QueryRequest_DurabilityLevel) Enum() *QueryRequest_DurabilityLevel {
	p := new(QueryRequest_DurabilityLevel)
	*p = x
	return p
}

func (x QueryRequest_DurabilityLevel) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (QueryRequest_DurabilityLevel) Descriptor() protoreflect.EnumDescriptor {
	return file_couchbase_query_v1_query_proto_enumTypes[1].Descriptor()
}

func (QueryRequest_DurabilityLevel) Type() protoreflect.EnumType {
	return &file_couchbase_query_v1_query_proto_enumTypes[1]
}

func (x QueryRequest_DurabilityLevel) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use QueryRequest_DurabilityLevel.Descriptor instead.
func (QueryRequest_DurabilityLevel) EnumDescriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{0, 1}
}

type QueryRequest_ProfileMode int32

const (
	QueryRequest_PROFILE_MODE_OFF     QueryRequest_ProfileMode = 0
	QueryRequest_PROFILE_MODE_PHASES  QueryRequest_ProfileMode = 1
	QueryRequest_PROFILE_MODE_TIMINGS QueryRequest_ProfileMode = 2
)

// Enum value maps for QueryRequest_ProfileMode.
var (
	QueryRequest_ProfileMode_name = map[int32]string{
		0: "PROFILE_MODE_OFF",
		1: "PROFILE_MODE_PHASES",
		2: "PROFILE_MODE_TIMINGS",
	}
	QueryRequest_ProfileMode_value = map[string]int32{
		"PROFILE_MODE_OFF":     0,
		"PROFILE_MODE_PHASES":  1,
		"PROFILE_MODE_TIMINGS": 2,
	}
)

func (x QueryRequest_ProfileMode) Enum() *QueryRequest_ProfileMode {
	p := new(QueryRequest_ProfileMode)
	*p = x
	return p
}

func (x QueryRequest_ProfileMode) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (QueryRequest_ProfileMode) Descriptor() protoreflect.EnumDescriptor {
	return file_couchbase_query_v1_query_proto_enumTypes[2].Descriptor()
}

func (QueryRequest_ProfileMode) Type() protoreflect.EnumType {
	return &file_couchbase_query_v1_query_proto_enumTypes[2]
}

func (x QueryRequest_ProfileMode) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use QueryRequest_ProfileMode.Descriptor instead.
func (QueryRequest_ProfileMode) EnumDescriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{0, 2}
}

type QueryResponse_MetaData_Status int32

const (
	QueryResponse_MetaData_STATUS_RUNNING   QueryResponse_MetaData_Status = 0
	QueryResponse_MetaData_STATUS_SUCCESS   QueryResponse_MetaData_Status = 1
	QueryResponse_MetaData_STATUS_ERRORS    QueryResponse_MetaData_Status = 2
	QueryResponse_MetaData_STATUS_COMPLETED QueryResponse_MetaData_Status = 3
	QueryResponse_MetaData_STATUS_STOPPED   QueryResponse_MetaData_Status = 4
	QueryResponse_MetaData_STATUS_TIMEOUT   QueryResponse_MetaData_Status = 5
	QueryResponse_MetaData_STATUS_CLOSED    QueryResponse_MetaData_Status = 6
	QueryResponse_MetaData_STATUS_FATAL     QueryResponse_MetaData_Status = 7
	QueryResponse_MetaData_STATUS_ABORTED   QueryResponse_MetaData_Status = 8
	QueryResponse_MetaData_STATUS_UNKNOWN   QueryResponse_MetaData_Status = 9
)

// Enum value maps for QueryResponse_MetaData_Status.
var (
	QueryResponse_MetaData_Status_name = map[int32]string{
		0: "STATUS_RUNNING",
		1: "STATUS_SUCCESS",
		2: "STATUS_ERRORS",
		3: "STATUS_COMPLETED",
		4: "STATUS_STOPPED",
		5: "STATUS_TIMEOUT",
		6: "STATUS_CLOSED",
		7: "STATUS_FATAL",
		8: "STATUS_ABORTED",
		9: "STATUS_UNKNOWN",
	}
	QueryResponse_MetaData_Status_value = map[string]int32{
		"STATUS_RUNNING":   0,
		"STATUS_SUCCESS":   1,
		"STATUS_ERRORS":    2,
		"STATUS_COMPLETED": 3,
		"STATUS_STOPPED":   4,
		"STATUS_TIMEOUT":   5,
		"STATUS_CLOSED":    6,
		"STATUS_FATAL":     7,
		"STATUS_ABORTED":   8,
		"STATUS_UNKNOWN":   9,
	}
)

func (x QueryResponse_MetaData_Status) Enum() *QueryResponse_MetaData_Status {
	p := new(QueryResponse_MetaData_Status)
	*p = x
	return p
}

func (x QueryResponse_MetaData_Status) String() string {
	return protoimpl.X.EnumStringOf(x.Descriptor(), protoreflect.EnumNumber(x))
}

func (QueryResponse_MetaData_Status) Descriptor() protoreflect.EnumDescriptor {
	return file_couchbase_query_v1_query_proto_enumTypes[3].Descriptor()
}

func (QueryResponse_MetaData_Status) Type() protoreflect.EnumType {
	return &file_couchbase_query_v1_query_proto_enumTypes[3]
}

func (x QueryResponse_MetaData_Status) Number() protoreflect.EnumNumber {
	return protoreflect.EnumNumber(x)
}

// Deprecated: Use QueryResponse_MetaData_Status.Descriptor instead.
func (QueryResponse_MetaData_Status) EnumDescriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{1, 0, 0}
}

type QueryRequest struct {
	state                protoimpl.MessageState        `protogen:"open.v1"`
	BucketName           *string                       `protobuf:"bytes,1,opt,name=bucket_name,json=bucketName,proto3,oneof" json:"bucket_name,omitempty"`
	ScopeName            *string                       `protobuf:"bytes,2,opt,name=scope_name,json=scopeName,proto3,oneof" json:"scope_name,omitempty"`
	Statement            string                        `protobuf:"bytes,3,opt,name=statement,proto3" json:"statement,omitempty"`
	ReadOnly             *bool                         `protobuf:"varint,4,opt,name=read_only,json=readOnly,proto3,oneof" json:"read_only,omitempty"`
	Prepared             *bool                         `protobuf:"varint,5,opt,name=prepared,proto3,oneof" json:"prepared,omitempty"`
	TuningOptions        *QueryRequest_TuningOptions   `protobuf:"bytes,6,opt,name=tuning_options,json=tuningOptions,proto3,oneof" json:"tuning_options,omitempty"`
	ClientContextId      *string                       `protobuf:"bytes,7,opt,name=client_context_id,json=clientContextId,proto3,oneof" json:"client_context_id,omitempty"`
	ScanConsistency      *QueryRequest_ScanConsistency `protobuf:"varint,8,opt,name=scan_consistency,json=scanConsistency,proto3,enum=couchbase.query.v1.QueryRequest_ScanConsistency,oneof" json:"scan_consistency,omitempty"`
	PositionalParameters [][]byte                      `protobuf:"bytes,9,rep,name=positional_parameters,json=positionalParameters,proto3" json:"positional_parameters,omitempty"`
	NamedParameters      map[string][]byte             `protobuf:"bytes,10,rep,name=named_parameters,json=namedParameters,proto3" json:"named_parameters,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	FlexIndex            *bool                         `protobuf:"varint,11,opt,name=flex_index,json=flexIndex,proto3,oneof" json:"flex_index,omitempty"`
	PreserveExpiry       *bool                         `protobuf:"varint,12,opt,name=preserve_expiry,json=preserveExpiry,proto3,oneof" json:"preserve_expiry,omitempty"`
	ConsistentWith       []*kv_v1.MutationToken        `protobuf:"bytes,13,rep,name=consistent_with,json=consistentWith,proto3" json:"consistent_with,omitempty"`
	DurabilityLevel      *QueryRequest_DurabilityLevel `protobuf:"varint,15,opt,name=durability_level,json=durabilityLevel,proto3,enum=couchbase.query.v1.QueryRequest_DurabilityLevel,oneof" json:"durability_level,omitempty"`
	ProfileMode          *QueryRequest_ProfileMode     `protobuf:"varint,14,opt,name=profile_mode,json=profileMode,proto3,enum=couchbase.query.v1.QueryRequest_ProfileMode,oneof" json:"profile_mode,omitempty"`
	unknownFields        protoimpl.UnknownFields
	sizeCache            protoimpl.SizeCache
}

func (x *QueryRequest) Reset() {
	*x = QueryRequest{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryRequest) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryRequest) ProtoMessage() {}

func (x *QueryRequest) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryRequest.ProtoReflect.Descriptor instead.
func (*QueryRequest) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{0}
}

func (x *QueryRequest) GetBucketName() string {
	if x != nil && x.BucketName != nil {
		return *x.BucketName
	}
	return ""
}

func (x *QueryRequest) GetScopeName() string {
	if x != nil && x.ScopeName != nil {
		return *x.ScopeName
	}
	return ""
}

func (x *QueryRequest) GetStatement() string {
	if x != nil {
		return x.Statement
	}
	return ""
}

func (x *QueryRequest) GetReadOnly() bool {
	if x != nil && x.ReadOnly != nil {
		return *x.ReadOnly
	}
	return false
}

func (x *QueryRequest) GetPrepared() bool {
	if x != nil && x.Prepared != nil {
		return *x.Prepared
	}
	return false
}

func (x *QueryRequest) GetTuningOptions() *QueryRequest_TuningOptions {
	if x != nil {
		return x.TuningOptions
	}
	return nil
}

func (x *QueryRequest) GetClientContextId() string {
	if x != nil && x.ClientContextId != nil {
		return *x.ClientContextId
	}
	return ""
}

func (x *QueryRequest) GetScanConsistency() QueryRequest_ScanConsistency {
	if x != nil && x.ScanConsistency != nil {
		return *x.ScanConsistency
	}
	return QueryRequest_SCAN_CONSISTENCY_NOT_BOUNDED
}

func (x *QueryRequest) GetPositionalParameters() [][]byte {
	if x != nil {
		return x.PositionalParameters
	}
	return nil
}

func (x *QueryRequest) GetNamedParameters() map[string][]byte {
	if x != nil {
		return x.NamedParameters
	}
	return nil
}

func (x *QueryRequest) GetFlexIndex() bool {
	if x != nil && x.FlexIndex != nil {
		return *x.FlexIndex
	}
	return false
}

func (x *QueryRequest) GetPreserveExpiry() bool {
	if x != nil && x.PreserveExpiry != nil {
		return *x.PreserveExpiry
	}
	return false
}

func (x *QueryRequest) GetConsistentWith() []*kv_v1.MutationToken {
	if x != nil {
		return x.ConsistentWith
	}
	return nil
}

func (x *QueryRequest) GetDurabilityLevel() QueryRequest_DurabilityLevel {
	if x != nil && x.DurabilityLevel != nil {
		return *x.DurabilityLevel
	}
	return QueryRequest_DURABILITY_LEVEL_NONE
}

func (x *QueryRequest) GetProfileMode() QueryRequest_ProfileMode {
	if x != nil && x.ProfileMode != nil {
		return *x.ProfileMode
	}
	return QueryRequest_PROFILE_MODE_OFF
}

type QueryResponse struct {
	state         protoimpl.MessageState  `protogen:"open.v1"`
	Rows          [][]byte                `protobuf:"bytes,1,rep,name=rows,proto3" json:"rows,omitempty"`
	MetaData      *QueryResponse_MetaData `protobuf:"bytes,2,opt,name=meta_data,json=metaData,proto3,oneof" json:"meta_data,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *QueryResponse) Reset() {
	*x = QueryResponse{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryResponse) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryResponse) ProtoMessage() {}

func (x *QueryResponse) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryResponse.ProtoReflect.Descriptor instead.
func (*QueryResponse) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{1}
}

func (x *QueryResponse) GetRows() [][]byte {
	if x != nil {
		return x.Rows
	}
	return nil
}

func (x *QueryResponse) GetMetaData() *QueryResponse_MetaData {
	if x != nil {
		return x.MetaData
	}
	return nil
}

type QueryRequest_TuningOptions struct {
	state          protoimpl.MessageState `protogen:"open.v1"`
	MaxParallelism *uint32                `protobuf:"varint,1,opt,name=max_parallelism,json=maxParallelism,proto3,oneof" json:"max_parallelism,omitempty"`
	PipelineBatch  *uint32                `protobuf:"varint,2,opt,name=pipeline_batch,json=pipelineBatch,proto3,oneof" json:"pipeline_batch,omitempty"`
	PipelineCap    *uint32                `protobuf:"varint,3,opt,name=pipeline_cap,json=pipelineCap,proto3,oneof" json:"pipeline_cap,omitempty"`
	ScanWait       *durationpb.Duration   `protobuf:"bytes,4,opt,name=scan_wait,json=scanWait,proto3,oneof" json:"scan_wait,omitempty"`
	ScanCap        *uint32                `protobuf:"varint,5,opt,name=scan_cap,json=scanCap,proto3,oneof" json:"scan_cap,omitempty"`
	DisableMetrics *bool                  `protobuf:"varint,6,opt,name=disable_metrics,json=disableMetrics,proto3,oneof" json:"disable_metrics,omitempty"`
	unknownFields  protoimpl.UnknownFields
	sizeCache      protoimpl.SizeCache
}

func (x *QueryRequest_TuningOptions) Reset() {
	*x = QueryRequest_TuningOptions{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryRequest_TuningOptions) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryRequest_TuningOptions) ProtoMessage() {}

func (x *QueryRequest_TuningOptions) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryRequest_TuningOptions.ProtoReflect.Descriptor instead.
func (*QueryRequest_TuningOptions) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{0, 0}
}

func (x *QueryRequest_TuningOptions) GetMaxParallelism() uint32 {
	if x != nil && x.MaxParallelism != nil {
		return *x.MaxParallelism
	}
	return 0
}

func (x *QueryRequest_TuningOptions) GetPipelineBatch() uint32 {
	if x != nil && x.PipelineBatch != nil {
		return *x.PipelineBatch
	}
	return 0
}

func (x *QueryRequest_TuningOptions) GetPipelineCap() uint32 {
	if x != nil && x.PipelineCap != nil {
		return *x.PipelineCap
	}
	return 0
}

func (x *QueryRequest_TuningOptions) GetScanWait() *durationpb.Duration {
	if x != nil {
		return x.ScanWait
	}
	return nil
}

func (x *QueryRequest_TuningOptions) GetScanCap() uint32 {
	if x != nil && x.ScanCap != nil {
		return *x.ScanCap
	}
	return 0
}

func (x *QueryRequest_TuningOptions) GetDisableMetrics() bool {
	if x != nil && x.DisableMetrics != nil {
		return *x.DisableMetrics
	}
	return false
}

type QueryResponse_MetaData struct {
	state           protoimpl.MessageState            `protogen:"open.v1"`
	RequestId       string                            `protobuf:"bytes,1,opt,name=request_id,json=requestId,proto3" json:"request_id,omitempty"`
	ClientContextId string                            `protobuf:"bytes,2,opt,name=client_context_id,json=clientContextId,proto3" json:"client_context_id,omitempty"`
	Metrics         *QueryResponse_MetaData_Metrics   `protobuf:"bytes,3,opt,name=metrics,proto3,oneof" json:"metrics,omitempty"`
	Status          QueryResponse_MetaData_Status     `protobuf:"varint,4,opt,name=status,proto3,enum=couchbase.query.v1.QueryResponse_MetaData_Status" json:"status,omitempty"`
	Warnings        []*QueryResponse_MetaData_Warning `protobuf:"bytes,5,rep,name=warnings,proto3" json:"warnings,omitempty"`
	Profile         []byte                            `protobuf:"bytes,6,opt,name=profile,proto3,oneof" json:"profile,omitempty"`
	Signature       []byte                            `protobuf:"bytes,7,opt,name=signature,proto3" json:"signature,omitempty"`
	unknownFields   protoimpl.UnknownFields
	sizeCache       protoimpl.SizeCache
}

func (x *QueryResponse_MetaData) Reset() {
	*x = QueryResponse_MetaData{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[4]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryResponse_MetaData) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryResponse_MetaData) ProtoMessage() {}

func (x *QueryResponse_MetaData) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[4]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryResponse_MetaData.ProtoReflect.Descriptor instead.
func (*QueryResponse_MetaData) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{1, 0}
}

func (x *QueryResponse_MetaData) GetRequestId() string {
	if x != nil {
		return x.RequestId
	}
	return ""
}

func (x *QueryResponse_MetaData) GetClientContextId() string {
	if x != nil {
		return x.ClientContextId
	}
	return ""
}

func (x *QueryResponse_MetaData) GetMetrics() *QueryResponse_MetaData_Metrics {
	if x != nil {
		return x.Metrics
	}
	return nil
}

func (x *QueryResponse_MetaData) GetStatus() QueryResponse_MetaData_Status {
	if x != nil {
		return x.Status
	}
	return QueryResponse_MetaData_STATUS_RUNNING
}

func (x *QueryResponse_MetaData) GetWarnings() []*QueryResponse_MetaData_Warning {
	if x != nil {
		return x.Warnings
	}
	return nil
}

func (x *QueryResponse_MetaData) GetProfile() []byte {
	if x != nil {
		return x.Profile
	}
	return nil
}

func (x *QueryResponse_MetaData) GetSignature() []byte {
	if x != nil {
		return x.Signature
	}
	return nil
}

type QueryResponse_MetaData_Warning struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Code          uint32                 `protobuf:"varint,1,opt,name=code,proto3" json:"code,omitempty"`
	Message       string                 `protobuf:"bytes,2,opt,name=message,proto3" json:"message,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *QueryResponse_MetaData_Warning) Reset() {
	*x = QueryResponse_MetaData_Warning{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[5]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryResponse_MetaData_Warning) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryResponse_MetaData_Warning) ProtoMessage() {}

func (x *QueryResponse_MetaData_Warning) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[5]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryResponse_MetaData_Warning.ProtoReflect.Descriptor instead.
func (*QueryResponse_MetaData_Warning) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{1, 0, 0}
}

func (x *QueryResponse_MetaData_Warning) GetCode() uint32 {
	if x != nil {
		return x.Code
	}
	return 0
}

func (x *QueryResponse_MetaData_Warning) GetMessage() string {
	if x != nil {
		return x.Message
	}
	return ""
}

type QueryResponse_MetaData_Metrics struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	ElapsedTime   *durationpb.Duration   `protobuf:"bytes,1,opt,name=elapsed_time,json=elapsedTime,proto3" json:"elapsed_time,omitempty"`
	ExecutionTime *durationpb.Duration   `protobuf:"bytes,2,opt,name=execution_time,json=executionTime,proto3" json:"execution_time,omitempty"`
	ResultCount   uint64                 `protobuf:"varint,3,opt,name=result_count,json=resultCount,proto3" json:"result_count,omitempty"`
	ResultSize    uint64                 `protobuf:"varint,4,opt,name=result_size,json=resultSize,proto3" json:"result_size,omitempty"`
	MutationCount uint64                 `protobuf:"varint,5,opt,name=mutation_count,json=mutationCount,proto3" json:"mutation_count,omitempty"`
	SortCount     uint64                 `protobuf:"varint,6,opt,name=sort_count,json=sortCount,proto3" json:"sort_count,omitempty"`
	ErrorCount    uint64                 `protobuf:"varint,7,opt,name=error_count,json=errorCount,proto3" json:"error_count,omitempty"`
	WarningCount  uint64                 `protobuf:"varint,8,opt,name=warning_count,json=warningCount,proto3" json:"warning_count,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *QueryResponse_MetaData_Metrics) Reset() {
	*x = QueryResponse_MetaData_Metrics{}
	mi := &file_couchbase_query_v1_query_proto_msgTypes[6]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *QueryResponse_MetaData_Metrics) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*QueryResponse_MetaData_Metrics) ProtoMessage() {}

func (x *QueryResponse_MetaData_Metrics) ProtoReflect() protoreflect.Message {
	mi := &file_couchbase_query_v1_query_proto_msgTypes[6]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use QueryResponse_MetaData_Metrics.ProtoReflect.Descriptor instead.
func (*QueryResponse_MetaData_Metrics) Descriptor() ([]byte, []int) {
	return file_couchbase_query_v1_query_proto_rawDescGZIP(), []int{1, 0, 1}
}

func (x *QueryResponse_MetaData_Metrics) GetElapsedTime() *durationpb.Duration {
	if x != nil {
		return x.ElapsedTime
	}
	return nil
}

func (x *QueryResponse_MetaData_Metrics) GetExecutionTime() *durationpb.Duration {
	if x != nil {
		return x.ExecutionTime
	}
	return nil
}

func (x *QueryResponse_MetaData_Metrics) GetResultCount() uint64 {
	if x != nil {
		return x.ResultCount
	}
	return 0
}

func (x *QueryResponse_MetaData_Metrics) GetResultSize() uint64 {
	if x != nil {
		return x.ResultSize
	}
	return 0
}

func (x *QueryResponse_MetaData_Metrics) GetMutationCount() uint64 {
	if x != nil {
		return x.MutationCount
	}
	return 0
}

func (x *QueryResponse_MetaData_Metrics) GetSortCount() uint64 {
	if x != nil {
		return x.SortCount
	}
	return 0
}

func (x *QueryResponse_MetaData_Metrics) GetErrorCount() uint64 {
	if x != nil {
		return x.ErrorCount
	}
	return 0
}

func (x *QueryResponse_MetaData_Metrics) GetWarningCount() uint64 {
	if x != nil {
		return x.WarningCount
	}
	return 0
}

var File_couchbase_query_v1_query_proto protoreflect.FileDescriptor

const file_couchbase_query_v1_query_proto_rawDesc = "" +
	"\n" +
	"\x1ecouchbase/query/v1/query.proto\x12\x12couchbase.query.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x18couchbase/kv/v1/kv.proto\"\xfa\x0e\n" +
	"\fQueryRequest\x12$\n" +
	"\vbucket_name\x18\x01 \x01(\tH\x00R\n" +
	"bucketName\x88\x01\x01\x12\"\n" +
	"\n" +
	"scope_name\x18\x02 \x01(\tH\x01R\tscopeName\x88\x01\x01\x12\x1c\n" +
	"\tstatement\x18\x03 \x01(\tR\tstatement\x12 \n" +
	"\tread_only\x18\x04 \x01(\bH\x02R\breadOnly\x88\x01\x01\x12\x1f\n" +
	"\bprepared\x18\x05 \x01(\bH\x03R\bprepared\x88\x01\x01\x12Z\n" +
	"\x0etuning_options\x18\x06 \x01(\v2..couchbase.query.v1.QueryRequest.TuningOptionsH\x04R\rtuningOptions\x88\x01\x01\x12/\n" +
	"\x11client_context_id\x18\a \x01(\tH\x05R\x0fclientContextId\x88\x01\x01\x12`\n" +
	"\x10scan_consistency\x18\b \x01(\x0e20.couchbase.query.v1.QueryRequest.ScanConsistencyH\x06R\x0fscanConsistency\x88\x01\x01\x123\n" +
	"\x15positional_parameters\x18\t \x03(\fR\x14positionalParameters\x12`\n" +
	"\x10named_parameters\x18\n" +
	" \x03(\v25.couchbase.query.v1.QueryRequest.NamedParametersEntryR\x0fnamedParameters\x12\"\n" +
	"\n" +
	"flex_index\x18\v \x01(\bH\aR\tflexIndex\x88\x01\x01\x12,\n" +
	"\x0fpreserve_expiry\x18\f \x01(\bH\bR\x0epreserveExpiry\x88\x01\x01\x12G\n" +
	"\x0fconsistent_with\x18\r \x03(\v2\x1e.couchbase.kv.v1.MutationTokenR\x0econsistentWith\x12`\n" +
	"\x10durability_level\x18\x0f \x01(\x0e20.couchbase.query.v1.QueryRequest.DurabilityLevelH\tR\x0fdurabilityLevel\x88\x01\x01\x12T\n" +
	"\fprofile_mode\x18\x0e \x01(\x0e2,.couchbase.query.v1.QueryRequest.ProfileModeH\n" +
	"R\vprofileMode\x88\x01\x01\x1a\x83\x03\n" +
	"\rTuningOptions\x12,\n" +
	"\x0fmax_parallelism\x18\x01 \x01(\rH\x00R\x0emaxParallelism\x88\x01\x01\x12*\n" +
	"\x0epipeline_batch\x18\x02 \x01(\rH\x01R\rpipelineBatch\x88\x01\x01\x12&\n" +
	"\fpipeline_cap\x18\x03 \x01(\rH\x02R\vpipelineCap\x88\x01\x01\x12;\n" +
	"\tscan_wait\x18\x04 \x01(\v2\x19.google.protobuf.DurationH\x03R\bscanWait\x88\x01\x01\x12\x1e\n" +
	"\bscan_cap\x18\x05 \x01(\rH\x04R\ascanCap\x88\x01\x01\x12,\n" +
	"\x0fdisable_metrics\x18\x06 \x01(\bH\x05R\x0edisableMetrics\x88\x01\x01B\x12\n" +
	"\x10_max_parallelismB\x11\n" +
	"\x0f_pipeline_batchB\x0f\n" +
	"\r_pipeline_capB\f\n" +
	"\n" +
	"_scan_waitB\v\n" +
	"\t_scan_capB\x12\n" +
	"\x10_disable_metrics\x1aB\n" +
	"\x14NamedParametersEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\fR\x05value:\x028\x01\"V\n" +
	"\x0fScanConsistency\x12 \n" +
	"\x1cSCAN_CONSISTENCY_NOT_BOUNDED\x10\x00\x12!\n" +
	"\x1dSCAN_CONSISTENCY_REQUEST_PLUS\x10\x01\"\xaa\x01\n" +
	"\x0fDurabilityLevel\x12\x19\n" +
	"\x15DURABILITY_LEVEL_NONE\x10\x00\x12\x1d\n" +
	"\x19DURABILITY_LEVEL_MAJORITY\x10\x01\x123\n" +
	"/DURABILITY_LEVEL_MAJORITY_AND_PERSIST_TO_ACTIVE\x10\x02\x12(\n" +
	"$DURABILITY_LEVEL_PERSIST_TO_MAJORITY\x10\x03\"V\n" +
	"\vProfileMode\x12\x14\n" +
	"\x10PROFILE_MODE_OFF\x10\x00\x12\x17\n" +
	"\x13PROFILE_MODE_PHASES\x10\x01\x12\x18\n" +
	"\x14PROFILE_MODE_TIMINGS\x10\x02B\x0e\n" +
	"\f_bucket_nameB\r\n" +
	"\v_scope_nameB\f\n" +
	"\n" +
	"_read_onlyB\v\n" +
	"\t_preparedB\x11\n" +
	"\x0f_tuning_optionsB\x14\n" +
	"\x12_client_context_idB\x13\n" +
	"\x11_scan_consistencyB\r\n" +
	"\v_flex_indexB\x12\n" +
	"\x10_preserve_expiryB\x13\n" +
	"\x11_durability_levelB\x0f\n" +
	"\r_profile_mode\"\x80\t\n" +
	"\rQueryResponse\x12\x12\n" +
	"\x04rows\x18\x01 \x03(\fR\x04rows\x12L\n" +
	"\tmeta_data\x18\x02 \x01(\v2*.couchbase.query.v1.QueryResponse.MetaDataH\x00R\bmetaData\x88\x01\x01\x1a\xfe\a\n" +
	"\bMetaData\x12\x1d\n" +
	"\n" +
	"request_id\x18\x01 \x01(\tR\trequestId\x12*\n" +
	"\x11client_context_id\x18\x02 \x01(\tR\x0fclientContextId\x12Q\n" +
	"\ametrics\x18\x03 \x01(\v22.couchbase.query.v1.QueryResponse.MetaData.MetricsH\x00R\ametrics\x88\x01\x01\x12I\n" +
	"\x06status\x18\x04 \x01(\x0e21.couchbase.query.v1.QueryResponse.MetaData.StatusR\x06status\x12N\n" +
	"\bwarnings\x18\x05 \x03(\v22.couchbase.query.v1.QueryResponse.MetaData.WarningR\bwarnings\x12\x1d\n" +
	"\aprofile\x18\x06 \x01(\fH\x01R\aprofile\x88\x01\x01\x12\x1c\n" +
	"\tsignature\x18\a \x01(\fR\tsignature\x1a7\n" +
	"\aWarning\x12\x12\n" +
	"\x04code\x18\x01 \x01(\rR\x04code\x12\x18\n" +
	"\amessage\x18\x02 \x01(\tR\amessage\x1a\xd9\x02\n" +
	"\aMetrics\x12<\n" +
	"\felapsed_time\x18\x01 \x01(\v2\x19.google.protobuf.DurationR\velapsedTime\x12@\n" +
	"\x0eexecution_time\x18\x02 \x01(\v2\x19.google.protobuf.DurationR\rexecutionTime\x12!\n" +
	"\fresult_count\x18\x03 \x01(\x04R\vresultCount\x12\x1f\n" +
	"\vresult_size\x18\x04 \x01(\x04R\n" +
	"resultSize\x12%\n" +
	"\x0emutation_count\x18\x05 \x01(\x04R\rmutationCount\x12\x1d\n" +
	"\n" +
	"sort_count\x18\x06 \x01(\x04R\tsortCount\x12\x1f\n" +
	"\verror_count\x18\a \x01(\x04R\n" +
	"errorCount\x12#\n" +
	"\rwarning_count\x18\b \x01(\x04R\fwarningCount\"\xce\x01\n" +
	"\x06Status\x12\x12\n" +
	"\x0eSTATUS_RUNNING\x10\x00\x12\x12\n" +
	"\x0eSTATUS_SUCCESS\x10\x01\x12\x11\n" +
	"\rSTATUS_ERRORS\x10\x02\x12\x14\n" +
	"\x10STATUS_COMPLETED\x10\x03\x12\x12\n" +
	"\x0eSTATUS_STOPPED\x10\x04\x12\x12\n" +
	"\x0eSTATUS_TIMEOUT\x10\x05\x12\x11\n" +
	"\rSTATUS_CLOSED\x10\x06\x12\x10\n" +
	"\fSTATUS_FATAL\x10\a\x12\x12\n" +
	"\x0eSTATUS_ABORTED\x10\b\x12\x12\n" +
	"\x0eSTATUS_UNKNOWN\x10\tB\n" +
	"\n" +
	"\b_metricsB\n" +
	"\n" +
	"\b_profileB\f\n" +
	"\n" +
	"_meta_data2`\n" +
	"\fQueryService\x12P\n" +
	"\x05Query\x12 .couchbase.query.v1.QueryRequest\x1a!.couchbase.query.v1.QueryResponse\"\x000\x01B\xec\x01\n" +
	"*com.couchbase.client.protostellar.query.v1P\x01Z>github.com/couchbase/goprotostellar/genproto/query_v1;query_v1\xaa\x02\x1fCouchbase.Protostellar.Query.V1\xca\x02)Couchbase\\Protostellar\\Generated\\Query\\V1\xea\x02-Couchbase::Protostellar::Generated::Query::V1b\x06proto3"

var (
	file_couchbase_query_v1_query_proto_rawDescOnce sync.Once
	file_couchbase_query_v1_query_proto_rawDescData []byte
)

func file_couchbase_query_v1_query_proto_rawDescGZIP() []byte {
	file_couchbase_query_v1_query_proto_rawDescOnce.Do(func() {
		file_couchbase_query_v1_query_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_couchbase_query_v1_query_proto_rawDesc), len(file_couchbase_query_v1_query_proto_rawDesc)))
	})
	return file_couchbase_query_v1_query_proto_rawDescData
}

var file_couchbase_query_v1_query_proto_enumTypes = make([]protoimpl.EnumInfo, 4)
var file_couchbase_query_v1_query_proto_msgTypes = make([]protoimpl.MessageInfo, 7)
var file_couchbase_query_v1_query_proto_goTypes = []any{
	(QueryRequest_ScanConsistency)(0),      // 0: couchbase.query.v1.QueryRequest.ScanConsistency
	(QueryRequest_DurabilityLevel)(0),      // 1: couchbase.query.v1.QueryRequest.DurabilityLevel
	(QueryRequest_ProfileMode)(0),          // 2: couchbase.query.v1.QueryRequest.ProfileMode
	(QueryResponse_MetaData_Status)(0),     // 3: couchbase.query.v1.QueryResponse.MetaData.Status
	(*QueryRequest)(nil),                   // 4: couchbase.query.v1.QueryRequest
	(*QueryResponse)(nil),                  // 5: couchbase.query.v1.QueryResponse
	(*QueryRequest_TuningOptions)(nil),     // 6: couchbase.query.v1.QueryRequest.TuningOptions
	nil,                                    // 7: couchbase.query.v1.QueryRequest.NamedParametersEntry
	(*QueryResponse_MetaData)(nil),         // 8: couchbase.query.v1.QueryResponse.MetaData
	(*QueryResponse_MetaData_Warning)(nil), // 9: couchbase.query.v1.QueryResponse.MetaData.Warning
	(*QueryResponse_MetaData_Metrics)(nil), // 10: couchbase.query.v1.QueryResponse.MetaData.Metrics
	(*kv_v1.MutationToken)(nil),            // 11: couchbase.kv.v1.MutationToken
	(*durationpb.Duration)(nil),            // 12: google.protobuf.Duration
}
var file_couchbase_query_v1_query_proto_depIdxs = []int32{
	6,  // 0: couchbase.query.v1.QueryRequest.tuning_options:type_name -> couchbase.query.v1.QueryRequest.TuningOptions
	0,  // 1: couchbase.query.v1.QueryRequest.scan_consistency:type_name -> couchbase.query.v1.QueryRequest.ScanConsistency
	7,  // 2: couchbase.query.v1.QueryRequest.named_parameters:type_name -> couchbase.query.v1.QueryRequest.NamedParametersEntry
	11, // 3: couchbase.query.v1.QueryRequest.consistent_with:type_name -> couchbase.kv.v1.MutationToken
	1,  // 4: couchbase.query.v1.QueryRequest.durability_level:type_name -> couchbase.query.v1.QueryRequest.DurabilityLevel
	2,  // 5: couchbase.query.v1.QueryRequest.profile_mode:type_name -> couchbase.query.v1.QueryRequest.ProfileMode
	8,  // 6: couchbase.query.v1.QueryResponse.meta_data:type_name -> couchbase.query.v1.QueryResponse.MetaData
	12, // 7: couchbase.query.v1.QueryRequest.TuningOptions.scan_wait:type_name -> google.protobuf.Duration
	10, // 8: couchbase.query.v1.QueryResponse.MetaData.metrics:type_name -> couchbase.query.v1.QueryResponse.MetaData.Metrics
	3,  // 9: couchbase.query.v1.QueryResponse.MetaData.status:type_name -> couchbase.query.v1.QueryResponse.MetaData.Status
	9,  // 10: couchbase.query.v1.QueryResponse.MetaData.warnings:type_name -> couchbase.query.v1.QueryResponse.MetaData.Warning
	12, // 11: couchbase.query.v1.QueryResponse.MetaData.Metrics.elapsed_time:type_name -> google.protobuf.Duration
	12, // 12: couchbase.query.v1.QueryResponse.MetaData.Metrics.execution_time:type_name -> google.protobuf.Duration
	4,  // 13: couchbase.query.v1.QueryService.Query:input_type -> couchbase.query.v1.QueryRequest
	5,  // 14: couchbase.query.v1.QueryService.Query:output_type -> couchbase.query.v1.QueryResponse
	14, // [14:15] is the sub-list for method output_type
	13, // [13:14] is the sub-list for method input_type
	13, // [13:13] is the sub-list for extension type_name
	13, // [13:13] is the sub-list for extension extendee
	0,  // [0:13] is the sub-list for field type_name
}

func init() { file_couchbase_query_v1_query_proto_init() }
func file_couchbase_query_v1_query_proto_init() {
	if File_couchbase_query_v1_query_proto != nil {
		return
	}
	file_couchbase_query_v1_query_proto_msgTypes[0].OneofWrappers = []any{}
	file_couchbase_query_v1_query_proto_msgTypes[1].OneofWrappers = []any{}
	file_couchbase_query_v1_query_proto_msgTypes[2].OneofWrappers = []any{}
	file_couchbase_query_v1_query_proto_msgTypes[4].OneofWrappers = []any{}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_couchbase_query_v1_query_proto_rawDesc), len(file_couchbase_query_v1_query_proto_rawDesc)),
			NumEnums:      4,
			NumMessages:   7,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_couchbase_query_v1_query_proto_goTypes,
		DependencyIndexes: file_couchbase_query_v1_query_proto_depIdxs,
		EnumInfos:         file_couchbase_query_v1_query_proto_enumTypes,
		MessageInfos:      file_couchbase_query_v1_query_proto_msgTypes,
	}.Build()
	File_couchbase_query_v1_query_proto = out.File
	file_couchbase_query_v1_query_proto_goTypes = nil
	file_couchbase_query_v1_query_proto_depIdxs = nil
}
