// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamConfirmMFAEnrollmentResponse hashicorp cloud iam confirm m f a enrollment response
//
// swagger:model hashicorp.cloud.iam.ConfirmMFAEnrollmentResponse
type HashicorpCloudIamConfirmMFAEnrollmentResponse interface{}
