// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamDisableMFAResponse hashicorp cloud iam disable m f a response
//
// swagger:model hashicorp.cloud.iam.DisableMFAResponse
type HashicorpCloudIamDisableMFAResponse interface{}
