// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"
	"strconv"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudIamCountGroupsForPrincipalsResponse hashicorp cloud iam count groups for principals response
//
// swagger:model hashicorp.cloud.iam.CountGroupsForPrincipalsResponse
type HashicorpCloudIamCountGroupsForPrincipalsResponse struct {

	// principals_counts is a list of principals and the number of groups that they are a member of.
	PrincipalsCounts []*HashicorpCloudIamCountGroupsForPrincipalsResponsePrincipalGroupCount `json:"principals_counts"`
}

// Validate validates this hashicorp cloud iam count groups for principals response
func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) Validate(formats strfmt.Registry) error {
	var res []error

	if err := m.validatePrincipalsCounts(formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) validatePrincipalsCounts(formats strfmt.Registry) error {
	if swag.IsZero(m.PrincipalsCounts) { // not required
		return nil
	}

	for i := 0; i < len(m.PrincipalsCounts); i++ {
		if swag.IsZero(m.PrincipalsCounts[i]) { // not required
			continue
		}

		if m.PrincipalsCounts[i] != nil {
			if err := m.PrincipalsCounts[i].Validate(formats); err != nil {
				if ve, ok := err.(*errors.Validation); ok {
					return ve.ValidateName("principals_counts" + "." + strconv.Itoa(i))
				} else if ce, ok := err.(*errors.CompositeError); ok {
					return ce.ValidateName("principals_counts" + "." + strconv.Itoa(i))
				}
				return err
			}
		}

	}

	return nil
}

// ContextValidate validate this hashicorp cloud iam count groups for principals response based on the context it is used
func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	var res []error

	if err := m.contextValidatePrincipalsCounts(ctx, formats); err != nil {
		res = append(res, err)
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) contextValidatePrincipalsCounts(ctx context.Context, formats strfmt.Registry) error {

	for i := 0; i < len(m.PrincipalsCounts); i++ {

		if m.PrincipalsCounts[i] != nil {

			if swag.IsZero(m.PrincipalsCounts[i]) { // not required
				return nil
			}

			if err := m.PrincipalsCounts[i].ContextValidate(ctx, formats); err != nil {
				if ve, ok := err.(*errors.Validation); ok {
					return ve.ValidateName("principals_counts" + "." + strconv.Itoa(i))
				} else if ce, ok := err.(*errors.CompositeError); ok {
					return ce.ValidateName("principals_counts" + "." + strconv.Itoa(i))
				}
				return err
			}
		}

	}

	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudIamCountGroupsForPrincipalsResponse) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudIamCountGroupsForPrincipalsResponse
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
