// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudResourcemanagerGetResourceResponse GetResourceResponse is the response for GetResource RPC
//
// swagger:model hashicorp.cloud.resourcemanager.GetResourceResponse
type HashicorpCloudResourcemanagerGetResourceResponse struct {

	// ParentId is the ID of the parent of the registered resource.
	ParentID string `json:"parent_id,omitempty"`

	// ResourceId is the ID of the registered resource.
	ResourceID string `json:"resource_id,omitempty"`

	// ResourceName is the name of the registered resource.
	ResourceName string `json:"resource_name,omitempty"`

	// ResourceType is the type of the registered resource.
	ResourceType string `json:"resource_type,omitempty"`
}

// Validate validates this hashicorp cloud resourcemanager get resource response
func (m *HashicorpCloudResourcemanagerGetResourceResponse) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this hashicorp cloud resourcemanager get resource response based on context it is used
func (m *HashicorpCloudResourcemanagerGetResourceResponse) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerGetResourceResponse) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerGetResourceResponse) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudResourcemanagerGetResourceResponse
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
