// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudResourcemanagerResourceV2 ResourceV2 is a container for resource v2 identifiers.
//
// swagger:model hashicorp.cloud.resourcemanager.ResourceV2
type HashicorpCloudResourcemanagerResourceV2 struct {

	// Geo is the geography where the resource is registered. A resource MAY NOT
	// have a geography associated with it. The currently valid values are: "",
	// "us", and "eu".
	Geo string `json:"geo,omitempty"`

	// ResourceID is the ID of the resource.
	ResourceID string `json:"resource_id,omitempty"`

	// ResourceName is the name of the resource.
	ResourceName string `json:"resource_name,omitempty"`
}

// Validate validates this hashicorp cloud resourcemanager resource v2
func (m *HashicorpCloudResourcemanagerResourceV2) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this hashicorp cloud resourcemanager resource v2 based on context it is used
func (m *HashicorpCloudResourcemanagerResourceV2) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerResourceV2) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerResourceV2) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudResourcemanagerResourceV2
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
