// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type PasskeyType int

const (
    DEVICEBOUND_PASSKEYTYPE PasskeyType = iota
    SYNCED_PASSKEYTYPE
    UNKNOWNFUTUREVALUE_PASSKEYTYPE
)

func (i PasskeyType) String() string {
    return []string{"deviceBound", "synced", "unknownFutureValue"}[i]
}
func ParsePasskeyType(v string) (any, error) {
    result := DEVICEBOUND_PASSKEYTYPE
    switch v {
        case "deviceBound":
            result = DEVICEBOUND_PASSKEYTYPE
        case "synced":
            result = SYNCED_PASSKEYTYPE
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_PASSKEYTYPE
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializePasskeyType(values []PasskeyType) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i PasskeyType) isMultiValue() bool {
    return false
}
