// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package externalconnectors
type ContentCategory int

const (
    UNCATEGORIZED_CONTENTCATEGORY ContentCategory = iota
    KNOWLEDGEBASE_CONTENTCATEGORY
    WIKIS_CONTENTCATEGORY
    FILEREPOSITORY_CONTENTCATEGORY
    QNA_CONTENTCATEGORY
    CRM_CONTENTCATEGORY
    DASHBOARD_CONTENTCATEGORY
    PEOPLE_CONTENTCATEGORY
    MEDIA_CONTENTCATEGORY
    EMAIL_CONTENTCATEGORY
    MESSAGING_CONTENTCATEGORY
    MEETINGTRANSCRIPTS_CONTENTCATEGORY
    TASKMANAGEMENT_CONTENTCATEGORY
    LEARNINGMANAGEMENT_CONTENTCATEGORY
    UNKNOWNFUTUREVALUE_CONTENTCATEGORY
)

func (i ContentCategory) String() string {
    return []string{"uncategorized", "knowledgeBase", "wikis", "fileRepository", "qna", "crm", "dashboard", "people", "media", "email", "messaging", "meetingTranscripts", "taskManagement", "learningManagement", "unknownFutureValue"}[i]
}
func ParseContentCategory(v string) (any, error) {
    result := UNCATEGORIZED_CONTENTCATEGORY
    switch v {
        case "uncategorized":
            result = UNCATEGORIZED_CONTENTCATEGORY
        case "knowledgeBase":
            result = KNOWLEDGEBASE_CONTENTCATEGORY
        case "wikis":
            result = WIKIS_CONTENTCATEGORY
        case "fileRepository":
            result = FILEREPOSITORY_CONTENTCATEGORY
        case "qna":
            result = QNA_CONTENTCATEGORY
        case "crm":
            result = CRM_CONTENTCATEGORY
        case "dashboard":
            result = DASHBOARD_CONTENTCATEGORY
        case "people":
            result = PEOPLE_CONTENTCATEGORY
        case "media":
            result = MEDIA_CONTENTCATEGORY
        case "email":
            result = EMAIL_CONTENTCATEGORY
        case "messaging":
            result = MESSAGING_CONTENTCATEGORY
        case "meetingTranscripts":
            result = MEETINGTRANSCRIPTS_CONTENTCATEGORY
        case "taskManagement":
            result = TASKMANAGEMENT_CONTENTCATEGORY
        case "learningManagement":
            result = LEARNINGMANAGEMENT_CONTENTCATEGORY
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_CONTENTCATEGORY
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeContentCategory(values []ContentCategory) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i ContentCategory) isMultiValue() bool {
    return false
}
