// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamDeleteOrganizationServicePrincipalResponse DeleteOrganizationServicePrincipalResponse is the response message returned after deleting
// a service principal on organization level.
//
// swagger:model hashicorp.cloud.iam.DeleteOrganizationServicePrincipalResponse
type HashicorpCloudIamDeleteOrganizationServicePrincipalResponse interface{}
