// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudResourcemanagerProjectGetProjectsCountResponse ProjectGetProjectsCountResponse see ProjectService.GetProjectsCount
//
// swagger:model hashicorp.cloud.resourcemanager.ProjectGetProjectsCountResponse
type HashicorpCloudResourcemanagerProjectGetProjectsCountResponse struct {

	// Count is the count of projects belonging to a parent scope.
	Count int32 `json:"count,omitempty"`
}

// Validate validates this hashicorp cloud resourcemanager project get projects count response
func (m *HashicorpCloudResourcemanagerProjectGetProjectsCountResponse) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this hashicorp cloud resourcemanager project get projects count response based on context it is used
func (m *HashicorpCloudResourcemanagerProjectGetProjectsCountResponse) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerProjectGetProjectsCountResponse) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudResourcemanagerProjectGetProjectsCountResponse) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudResourcemanagerProjectGetProjectsCountResponse
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
