// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"
	"encoding/json"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/validate"
)

// HashicorpCloudVaultLink20221107LinkedClusterNodeState hashicorp cloud vault link 20221107 linked cluster node state
//
// swagger:model hashicorp.cloud.vault_link_20221107.LinkedClusterNode.State
type HashicorpCloudVaultLink20221107LinkedClusterNodeState string

func NewHashicorpCloudVaultLink20221107LinkedClusterNodeState(value HashicorpCloudVaultLink20221107LinkedClusterNodeState) *HashicorpCloudVaultLink20221107LinkedClusterNodeState {
	return &value
}

// Pointer returns a pointer to a freshly-allocated HashicorpCloudVaultLink20221107LinkedClusterNodeState.
func (m HashicorpCloudVaultLink20221107LinkedClusterNodeState) Pointer() *HashicorpCloudVaultLink20221107LinkedClusterNodeState {
	return &m
}

const (

	// HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKEDCLUSTERSTATEINVALID captures enum value "LINKED_CLUSTER_STATE_INVALID"
	HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKEDCLUSTERSTATEINVALID HashicorpCloudVaultLink20221107LinkedClusterNodeState = "LINKED_CLUSTER_STATE_INVALID"

	// HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKING captures enum value "LINKING"
	HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKING HashicorpCloudVaultLink20221107LinkedClusterNodeState = "LINKING"

	// HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKED captures enum value "LINKED"
	HashicorpCloudVaultLink20221107LinkedClusterNodeStateLINKED HashicorpCloudVaultLink20221107LinkedClusterNodeState = "LINKED"

	// HashicorpCloudVaultLink20221107LinkedClusterNodeStateUNLINKING captures enum value "UNLINKING"
	HashicorpCloudVaultLink20221107LinkedClusterNodeStateUNLINKING HashicorpCloudVaultLink20221107LinkedClusterNodeState = "UNLINKING"

	// HashicorpCloudVaultLink20221107LinkedClusterNodeStateUNLINKED captures enum value "UNLINKED"
	HashicorpCloudVaultLink20221107LinkedClusterNodeStateUNLINKED HashicorpCloudVaultLink20221107LinkedClusterNodeState = "UNLINKED"
)

// for schema
var hashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum []interface{}

func init() {
	var res []HashicorpCloudVaultLink20221107LinkedClusterNodeState
	if err := json.Unmarshal([]byte(`["LINKED_CLUSTER_STATE_INVALID","LINKING","LINKED","UNLINKING","UNLINKED"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		hashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum = append(hashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum, v)
	}
}

func (m HashicorpCloudVaultLink20221107LinkedClusterNodeState) validateHashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum(path, location string, value HashicorpCloudVaultLink20221107LinkedClusterNodeState) error {
	if err := validate.EnumCase(path, location, value, hashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum, true); err != nil {
		return err
	}
	return nil
}

// Validate validates this hashicorp cloud vault link 20221107 linked cluster node state
func (m HashicorpCloudVaultLink20221107LinkedClusterNodeState) Validate(formats strfmt.Registry) error {
	var res []error

	// value enum
	if err := m.validateHashicorpCloudVaultLink20221107LinkedClusterNodeStateEnum("", "body", m); err != nil {
		return err
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

// ContextValidate validates this hashicorp cloud vault link 20221107 linked cluster node state based on context it is used
func (m HashicorpCloudVaultLink20221107LinkedClusterNodeState) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}
