// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamUpdateSSOConfigurationResponse hashicorp cloud iam update s s o configuration response
//
// swagger:model hashicorp.cloud.iam.UpdateSSOConfigurationResponse
type HashicorpCloudIamUpdateSSOConfigurationResponse interface{}
