// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type MonitorMode int

const (
    MONITORONLY_MONITORMODE MonitorMode = iota
    // A marker value for members added after the release of this API.
    UNKNOWNFUTUREVALUE_MONITORMODE
)

func (i MonitorMode) String() string {
    return []string{"monitorOnly", "unknownFutureValue"}[i]
}
func ParseMonitorMode(v string) (any, error) {
    result := MONITORONLY_MONITORMODE
    switch v {
        case "monitorOnly":
            result = MONITORONLY_MONITORMODE
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_MONITORMODE
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeMonitorMode(values []MonitorMode) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i MonitorMode) isMultiValue() bool {
    return false
}
