// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type CheckInMethod int

const (
    UNSPECIFIED_CHECKINMETHOD CheckInMethod = iota
    MANUAL_CHECKINMETHOD
    INFERRED_CHECKINMETHOD
    VERIFIED_CHECKINMETHOD
    UNKNOWNFUTUREVALUE_CHECKINMETHOD
)

func (i CheckInMethod) String() string {
    return []string{"unspecified", "manual", "inferred", "verified", "unknownFutureValue"}[i]
}
func ParseCheckInMethod(v string) (any, error) {
    result := UNSPECIFIED_CHECKINMETHOD
    switch v {
        case "unspecified":
            result = UNSPECIFIED_CHECKINMETHOD
        case "manual":
            result = MANUAL_CHECKINMETHOD
        case "inferred":
            result = INFERRED_CHECKINMETHOD
        case "verified":
            result = VERIFIED_CHECKINMETHOD
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_CHECKINMETHOD
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeCheckInMethod(values []CheckInMethod) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i CheckInMethod) isMultiValue() bool {
    return false
}
