// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type LabelActionSource int

const (
    MANUAL_LABELACTIONSOURCE LabelActionSource = iota
    AUTOMATIC_LABELACTIONSOURCE
    RECOMMENDED_LABELACTIONSOURCE
    NONE_LABELACTIONSOURCE
    UNKNOWNFUTUREVALUE_LABELACTIONSOURCE
)

func (i LabelActionSource) String() string {
    return []string{"manual", "automatic", "recommended", "none", "unknownFutureValue"}[i]
}
func ParseLabelActionSource(v string) (any, error) {
    result := MANUAL_LABELACTIONSOURCE
    switch v {
        case "manual":
            result = MANUAL_LABELACTIONSOURCE
        case "automatic":
            result = AUTOMATIC_LABELACTIONSOURCE
        case "recommended":
            result = RECOMMENDED_LABELACTIONSOURCE
        case "none":
            result = NONE_LABELACTIONSOURCE
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_LABELACTIONSOURCE
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeLabelActionSource(values []LabelActionSource) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i LabelActionSource) isMultiValue() bool {
    return false
}
