// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package admin

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
    ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e "github.com/microsoft/kiota-abstractions-go/store"
    i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84 "github.com/microsoftgraph/msgraph-sdk-go/models/teamsadministration"
)

type TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody struct {
    // Stores model information.
    backingStore ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStore
}
// NewTeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody instantiates a new TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody and sets the default values.
func NewTeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody()(*TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) {
    m := &TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody{
    }
    m.backingStore = ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStoreFactoryInstance();
    m.SetAdditionalData(make(map[string]any))
    return m
}
// CreateTeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBodyFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateTeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBodyFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewTeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody(), nil
}
// GetAdditionalData gets the AdditionalData property value. Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
// returns a map[string]any when successful
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) GetAdditionalData()(map[string]any) {
    val , err :=  m.backingStore.Get("additionalData")
    if err != nil {
        panic(err)
    }
    if val == nil {
        var value = make(map[string]any);
        m.SetAdditionalData(value);
    }
    return val.(map[string]any)
}
// GetBackingStore gets the BackingStore property value. Stores model information.
// returns a BackingStore when successful
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) GetBackingStore()(ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStore) {
    return m.backingStore
}
// GetFieldDeserializers the deserialization information for the current model
// returns a map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error) when successful
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) GetFieldDeserializers()(map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error)) {
    res := make(map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error))
    res["value"] = func (n i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode) error {
        val, err := n.GetCollectionOfObjectValues(i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.CreateTeamsPolicyUserAssignmentFromDiscriminatorValue)
        if err != nil {
            return err
        }
        if val != nil {
            res := make([]i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable, len(val))
            for i, v := range val {
                if v != nil {
                    res[i] = v.(i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable)
                }
            }
            m.SetValue(res)
        }
        return nil
    }
    return res
}
// GetValue gets the value property value. The value property
// returns a []TeamsPolicyUserAssignmentable when successful
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) GetValue()([]i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable) {
    val, err := m.GetBackingStore().Get("value")
    if err != nil {
        panic(err)
    }
    if val != nil {
        return val.([]i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable)
    }
    return nil
}
// Serialize serializes information the current object
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) Serialize(writer i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.SerializationWriter)(error) {
    if m.GetValue() != nil {
        cast := make([]i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, len(m.GetValue()))
        for i, v := range m.GetValue() {
            if v != nil {
                cast[i] = v.(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable)
            }
        }
        err := writer.WriteCollectionOfObjectValues("value", cast)
        if err != nil {
            return err
        }
    }
    {
        err := writer.WriteAdditionalData(m.GetAdditionalData())
        if err != nil {
            return err
        }
    }
    return nil
}
// SetAdditionalData sets the AdditionalData property value. Stores additional data not described in the OpenAPI description found when deserializing. Can be used for serialization as well.
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) SetAdditionalData(value map[string]any)() {
    err := m.GetBackingStore().Set("additionalData", value)
    if err != nil {
        panic(err)
    }
}
// SetBackingStore sets the BackingStore property value. Stores model information.
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) SetBackingStore(value ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStore)() {
    m.backingStore = value
}
// SetValue sets the value property value. The value property
func (m *TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBody) SetValue(value []i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable)() {
    err := m.GetBackingStore().Set("value", value)
    if err != nil {
        panic(err)
    }
}
type TeamsPolicyUserAssignmentsMicrosoftGraphTeamsAdministrationAssignAssignPostRequestBodyable interface {
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.AdditionalDataHolder
    ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackedModel
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
    GetBackingStore()(ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStore)
    GetValue()([]i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable)
    SetBackingStore(value ie8677ce2c7e1b4c22e9c3827ecd078d41185424dd9eeb92b7d971ed2d49a392e.BackingStore)()
    SetValue(value []i22ad7c1b3905b3ca3489583547f6edfaef2f77bdd9a38e24b2fdb52e37702f84.TeamsPolicyUserAssignmentable)()
}
