// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamDeleteOrganizationServicePrincipalKeyResponse DeleteOrganizationServicePrincipalKeyResponse is the response to a DeleteOrganizationServicePrincipalKeyRequest.
// It is blank as the mere presence of the response indicates a success, in all other cases
// the request returns an error.
//
// swagger:model hashicorp.cloud.iam.DeleteOrganizationServicePrincipalKeyResponse
type HashicorpCloudIamDeleteOrganizationServicePrincipalKeyResponse interface{}
