// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudVault20201125AddPluginResponse hashicorp cloud vault 20201125 add plugin response
//
// swagger:model hashicorp.cloud.vault_20201125.AddPluginResponse
type HashicorpCloudVault20201125AddPluginResponse interface{}
