// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models
type RestoreArtifactsBulkRequestStatus int

const (
    UNKNOWN_RESTOREARTIFACTSBULKREQUESTSTATUS RestoreArtifactsBulkRequestStatus = iota
    ACTIVE_RESTOREARTIFACTSBULKREQUESTSTATUS
    COMPLETED_RESTOREARTIFACTSBULKREQUESTSTATUS
    COMPLETEDWITHERRORS_RESTOREARTIFACTSBULKREQUESTSTATUS
    UNKNOWNFUTUREVALUE_RESTOREARTIFACTSBULKREQUESTSTATUS
)

func (i RestoreArtifactsBulkRequestStatus) String() string {
    return []string{"unknown", "active", "completed", "completedWithErrors", "unknownFutureValue"}[i]
}
func ParseRestoreArtifactsBulkRequestStatus(v string) (any, error) {
    result := UNKNOWN_RESTOREARTIFACTSBULKREQUESTSTATUS
    switch v {
        case "unknown":
            result = UNKNOWN_RESTOREARTIFACTSBULKREQUESTSTATUS
        case "active":
            result = ACTIVE_RESTOREARTIFACTSBULKREQUESTSTATUS
        case "completed":
            result = COMPLETED_RESTOREARTIFACTSBULKREQUESTSTATUS
        case "completedWithErrors":
            result = COMPLETEDWITHERRORS_RESTOREARTIFACTSBULKREQUESTSTATUS
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_RESTOREARTIFACTSBULKREQUESTSTATUS
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeRestoreArtifactsBulkRequestStatus(values []RestoreArtifactsBulkRequestStatus) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i RestoreArtifactsBulkRequestStatus) isMultiValue() bool {
    return false
}
