// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"
	"encoding/json"

	"github.com/go-openapi/errors"
	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/validate"
)

// HashicorpCloudVault20201125NewRelicRegion hashicorp cloud vault 20201125 new relic region
//
// swagger:model hashicorp.cloud.vault_20201125.NewRelic.Region
type HashicorpCloudVault20201125NewRelicRegion string

func NewHashicorpCloudVault20201125NewRelicRegion(value HashicorpCloudVault20201125NewRelicRegion) *HashicorpCloudVault20201125NewRelicRegion {
	return &value
}

// Pointer returns a pointer to a freshly-allocated HashicorpCloudVault20201125NewRelicRegion.
func (m HashicorpCloudVault20201125NewRelicRegion) Pointer() *HashicorpCloudVault20201125NewRelicRegion {
	return &m
}

const (

	// HashicorpCloudVault20201125NewRelicRegionNEWRELICREGIONINVALID captures enum value "NEW_RELIC_REGION_INVALID"
	HashicorpCloudVault20201125NewRelicRegionNEWRELICREGIONINVALID HashicorpCloudVault20201125NewRelicRegion = "NEW_RELIC_REGION_INVALID"

	// HashicorpCloudVault20201125NewRelicRegionUS captures enum value "US"
	HashicorpCloudVault20201125NewRelicRegionUS HashicorpCloudVault20201125NewRelicRegion = "US"

	// HashicorpCloudVault20201125NewRelicRegionEU captures enum value "EU"
	HashicorpCloudVault20201125NewRelicRegionEU HashicorpCloudVault20201125NewRelicRegion = "EU"
)

// for schema
var hashicorpCloudVault20201125NewRelicRegionEnum []interface{}

func init() {
	var res []HashicorpCloudVault20201125NewRelicRegion
	if err := json.Unmarshal([]byte(`["NEW_RELIC_REGION_INVALID","US","EU"]`), &res); err != nil {
		panic(err)
	}
	for _, v := range res {
		hashicorpCloudVault20201125NewRelicRegionEnum = append(hashicorpCloudVault20201125NewRelicRegionEnum, v)
	}
}

func (m HashicorpCloudVault20201125NewRelicRegion) validateHashicorpCloudVault20201125NewRelicRegionEnum(path, location string, value HashicorpCloudVault20201125NewRelicRegion) error {
	if err := validate.EnumCase(path, location, value, hashicorpCloudVault20201125NewRelicRegionEnum, true); err != nil {
		return err
	}
	return nil
}

// Validate validates this hashicorp cloud vault 20201125 new relic region
func (m HashicorpCloudVault20201125NewRelicRegion) Validate(formats strfmt.Registry) error {
	var res []error

	// value enum
	if err := m.validateHashicorpCloudVault20201125NewRelicRegionEnum("", "body", m); err != nil {
		return err
	}

	if len(res) > 0 {
		return errors.CompositeValidationError(res...)
	}
	return nil
}

// ContextValidate validates this hashicorp cloud vault 20201125 new relic region based on context it is used
func (m HashicorpCloudVault20201125NewRelicRegion) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}
