// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package security
type ServiceStatus int

const (
    STOPPED_SERVICESTATUS ServiceStatus = iota
    STARTING_SERVICESTATUS
    RUNNING_SERVICESTATUS
    DISABLED_SERVICESTATUS
    ONBOARDING_SERVICESTATUS
    UNKNOWN_SERVICESTATUS
    UNKNOWNFUTUREVALUE_SERVICESTATUS
)

func (i ServiceStatus) String() string {
    return []string{"stopped", "starting", "running", "disabled", "onboarding", "unknown", "unknownFutureValue"}[i]
}
func ParseServiceStatus(v string) (any, error) {
    result := STOPPED_SERVICESTATUS
    switch v {
        case "stopped":
            result = STOPPED_SERVICESTATUS
        case "starting":
            result = STARTING_SERVICESTATUS
        case "running":
            result = RUNNING_SERVICESTATUS
        case "disabled":
            result = DISABLED_SERVICESTATUS
        case "onboarding":
            result = ONBOARDING_SERVICESTATUS
        case "unknown":
            result = UNKNOWN_SERVICESTATUS
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_SERVICESTATUS
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeServiceStatus(values []ServiceStatus) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i ServiceStatus) isMultiValue() bool {
    return false
}
