// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// HashicorpCloudIamCreateOrganizationInvitationsResponse CreateOrganizationInvitationsResponse is a response for a created organization invitation.
//
// TODO: add list of the invitations that have been created.
//
// swagger:model hashicorp.cloud.iam.CreateOrganizationInvitationsResponse
type HashicorpCloudIamCreateOrganizationInvitationsResponse interface{}
