// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudVault20201125InputVaultInsightsConfig hashicorp cloud vault 20201125 input vault insights config
//
// swagger:model hashicorp.cloud.vault_20201125.InputVaultInsightsConfig
type HashicorpCloudVault20201125InputVaultInsightsConfig struct {

	// enabled controls the streaming of audit logs to Vault Insights
	Enabled bool `json:"enabled,omitempty"`
}

// Validate validates this hashicorp cloud vault 20201125 input vault insights config
func (m *HashicorpCloudVault20201125InputVaultInsightsConfig) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this hashicorp cloud vault 20201125 input vault insights config based on context it is used
func (m *HashicorpCloudVault20201125InputVaultInsightsConfig) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudVault20201125InputVaultInsightsConfig) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudVault20201125InputVaultInsightsConfig) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudVault20201125InputVaultInsightsConfig
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
