// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package directory

import (
    "context"
    i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f "github.com/microsoft/kiota-abstractions-go"
    ia572726a95efa92ddd544552cd950653dc691023836923576b2f4bf716cf204a "github.com/microsoftgraph/msgraph-sdk-go/models/odataerrors"
)

// PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder provides operations to call the upload method.
type PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder struct {
    i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.BaseRequestBuilder
}
// PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderPostRequestConfiguration configuration for the request such as headers, query parameters, and middleware options.
type PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderPostRequestConfiguration struct {
    // Request headers
    Headers *i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.RequestHeaders
    // Request options
    Options []i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.RequestOption
}
// NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderInternal instantiates a new PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder and sets the default values.
func NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderInternal(pathParameters map[string]string, requestAdapter i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.RequestAdapter)(*PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) {
    m := &PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder{
        BaseRequestBuilder: *i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.NewBaseRequestBuilder(requestAdapter, "{+baseurl}/directory/publicKeyInfrastructure/certificateBasedAuthConfigurations/{certificateBasedAuthPki%2Did}/upload", pathParameters),
    }
    return m
}
// NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder instantiates a new PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder and sets the default values.
func NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder(rawUrl string, requestAdapter i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.RequestAdapter)(*PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) {
    urlParams := make(map[string]string)
    urlParams["request-raw-url"] = rawUrl
    return NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderInternal(urlParams, requestAdapter)
}
// Post append additional certificate authority details to a certificateBasedAuthPki resource. Only one operation can run at a time and this operation can take up to 30 minutes to complete. To know whether another upload is in progress, call the Get certificateBasedAuthPki. The status property will have the value running.
// returns a ODataError error when the service returns a 4XX or 5XX status code
// [Find more info here]
// 
// [Find more info here]: https://learn.microsoft.com/graph/api/certificatebasedauthpki-upload?view=graph-rest-1.0
func (m *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) Post(ctx context.Context, body PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadPostRequestBodyable, requestConfiguration *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderPostRequestConfiguration)(error) {
    requestInfo, err := m.ToPostRequestInformation(ctx, body, requestConfiguration);
    if err != nil {
        return err
    }
    errorMapping := i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.ErrorMappings {
        "XXX": ia572726a95efa92ddd544552cd950653dc691023836923576b2f4bf716cf204a.CreateODataErrorFromDiscriminatorValue,
    }
    err = m.BaseRequestBuilder.RequestAdapter.SendNoContent(ctx, requestInfo, errorMapping)
    if err != nil {
        return err
    }
    return nil
}
// ToPostRequestInformation append additional certificate authority details to a certificateBasedAuthPki resource. Only one operation can run at a time and this operation can take up to 30 minutes to complete. To know whether another upload is in progress, call the Get certificateBasedAuthPki. The status property will have the value running.
// returns a *RequestInformation when successful
func (m *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) ToPostRequestInformation(ctx context.Context, body PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadPostRequestBodyable, requestConfiguration *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilderPostRequestConfiguration)(*i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.RequestInformation, error) {
    requestInfo := i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.NewRequestInformationWithMethodAndUrlTemplateAndPathParameters(i2ae4187f7daee263371cb1c977df639813ab50ffa529013b7437480d1ec0158f.POST, m.BaseRequestBuilder.UrlTemplate, m.BaseRequestBuilder.PathParameters)
    if requestConfiguration != nil {
        requestInfo.Headers.AddAll(requestConfiguration.Headers)
        requestInfo.AddRequestOptions(requestConfiguration.Options)
    }
    requestInfo.Headers.TryAdd("Accept", "application/json")
    err := requestInfo.SetContentFromParsable(ctx, m.BaseRequestBuilder.RequestAdapter, "application/json", body)
    if err != nil {
        return nil, err
    }
    return requestInfo, nil
}
// WithUrl returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
// returns a *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder when successful
func (m *PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) WithUrl(rawUrl string)(*PublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder) {
    return NewPublicKeyInfrastructureCertificateBasedAuthConfigurationsItemUploadRequestBuilder(rawUrl, m.BaseRequestBuilder.RequestAdapter);
}
