// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package security
type AiAgentPlatform int

const (
    UNKNOWN_AIAGENTPLATFORM AiAgentPlatform = iota
    AZUREAIFOUNDRY_AIAGENTPLATFORM
    COPILOTSTUDIO_AIAGENTPLATFORM
    COPILOT_AIAGENTPLATFORM
    UNKNOWNFUTUREVALUE_AIAGENTPLATFORM
)

func (i AiAgentPlatform) String() string {
    return []string{"unknown", "azureAIFoundry", "copilotStudio", "copilot", "unknownFutureValue"}[i]
}
func ParseAiAgentPlatform(v string) (any, error) {
    result := UNKNOWN_AIAGENTPLATFORM
    switch v {
        case "unknown":
            result = UNKNOWN_AIAGENTPLATFORM
        case "azureAIFoundry":
            result = AZUREAIFOUNDRY_AIAGENTPLATFORM
        case "copilotStudio":
            result = COPILOTSTUDIO_AIAGENTPLATFORM
        case "copilot":
            result = COPILOT_AIAGENTPLATFORM
        case "unknownFutureValue":
            result = UNKNOWNFUTUREVALUE_AIAGENTPLATFORM
        default:
            return nil, nil
    }
    return &result, nil
}
func SerializeAiAgentPlatform(values []AiAgentPlatform) []string {
    result := make([]string, len(values))
    for i, v := range values {
        result[i] = v.String()
    }
    return result
}
func (i AiAgentPlatform) isMultiValue() bool {
    return false
}
