// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package groups

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
)

// Deprecated: This class is obsolete. Use ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponseable instead.
type ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse struct {
    ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponse
}
// NewItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse instantiates a new ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse and sets the default values.
func NewItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse()(*ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse) {
    m := &ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse{
        ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponse: *NewItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponse(),
    }
    return m
}
// CreateItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponseFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponseFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponse(), nil
}
// Deprecated: This class is obsolete. Use ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponseable instead.
type ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksResponseable interface {
    ItemOnenoteNotebooksGetRecentNotebooksWithIncludePersonalNotebooksGetResponseable
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
}
