// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
)

type DaylightTimeZoneOffset struct {
    StandardTimeZoneOffset
}
// NewDaylightTimeZoneOffset instantiates a new DaylightTimeZoneOffset and sets the default values.
func NewDaylightTimeZoneOffset()(*DaylightTimeZoneOffset) {
    m := &DaylightTimeZoneOffset{
        StandardTimeZoneOffset: *NewStandardTimeZoneOffset(),
    }
    odataTypeValue := "#microsoft.graph.daylightTimeZoneOffset"
    m.SetOdataType(&odataTypeValue)
    return m
}
// CreateDaylightTimeZoneOffsetFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateDaylightTimeZoneOffsetFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewDaylightTimeZoneOffset(), nil
}
// GetDaylightBias gets the daylightBias property value. The time offset from Coordinated Universal Time (UTC) for daylight saving time. This value is in minutes.
// returns a *int32 when successful
func (m *DaylightTimeZoneOffset) GetDaylightBias()(*int32) {
    val, err := m.GetBackingStore().Get("daylightBias")
    if err != nil {
        panic(err)
    }
    if val != nil {
        return val.(*int32)
    }
    return nil
}
// GetFieldDeserializers the deserialization information for the current model
// returns a map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error) when successful
func (m *DaylightTimeZoneOffset) GetFieldDeserializers()(map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error)) {
    res := m.StandardTimeZoneOffset.GetFieldDeserializers()
    res["daylightBias"] = func (n i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode) error {
        val, err := n.GetInt32Value()
        if err != nil {
            return err
        }
        if val != nil {
            m.SetDaylightBias(val)
        }
        return nil
    }
    return res
}
// Serialize serializes information the current object
func (m *DaylightTimeZoneOffset) Serialize(writer i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.SerializationWriter)(error) {
    err := m.StandardTimeZoneOffset.Serialize(writer)
    if err != nil {
        return err
    }
    {
        err = writer.WriteInt32Value("daylightBias", m.GetDaylightBias())
        if err != nil {
            return err
        }
    }
    return nil
}
// SetDaylightBias sets the daylightBias property value. The time offset from Coordinated Universal Time (UTC) for daylight saving time. This value is in minutes.
func (m *DaylightTimeZoneOffset) SetDaylightBias(value *int32)() {
    err := m.GetBackingStore().Set("daylightBias", value)
    if err != nil {
        panic(err)
    }
}
type DaylightTimeZoneOffsetable interface {
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
    StandardTimeZoneOffsetable
    GetDaylightBias()(*int32)
    SetDaylightBias(value *int32)()
}
