// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
)

type FraudProtectionProviderConfiguration struct {
    FraudProtectionConfiguration
}
// NewFraudProtectionProviderConfiguration instantiates a new FraudProtectionProviderConfiguration and sets the default values.
func NewFraudProtectionProviderConfiguration()(*FraudProtectionProviderConfiguration) {
    m := &FraudProtectionProviderConfiguration{
        FraudProtectionConfiguration: *NewFraudProtectionConfiguration(),
    }
    odataTypeValue := "#microsoft.graph.fraudProtectionProviderConfiguration"
    m.SetOdataType(&odataTypeValue)
    return m
}
// CreateFraudProtectionProviderConfigurationFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateFraudProtectionProviderConfigurationFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewFraudProtectionProviderConfiguration(), nil
}
// GetFieldDeserializers the deserialization information for the current model
// returns a map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error) when successful
func (m *FraudProtectionProviderConfiguration) GetFieldDeserializers()(map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error)) {
    res := m.FraudProtectionConfiguration.GetFieldDeserializers()
    res["fraudProtectionProvider"] = func (n i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode) error {
        val, err := n.GetObjectValue(CreateFraudProtectionProviderFromDiscriminatorValue)
        if err != nil {
            return err
        }
        if val != nil {
            m.SetFraudProtectionProvider(val.(FraudProtectionProviderable))
        }
        return nil
    }
    return res
}
// GetFraudProtectionProvider gets the fraudProtectionProvider property value. The fraudProtectionProvider property
// returns a FraudProtectionProviderable when successful
func (m *FraudProtectionProviderConfiguration) GetFraudProtectionProvider()(FraudProtectionProviderable) {
    val, err := m.GetBackingStore().Get("fraudProtectionProvider")
    if err != nil {
        panic(err)
    }
    if val != nil {
        return val.(FraudProtectionProviderable)
    }
    return nil
}
// Serialize serializes information the current object
func (m *FraudProtectionProviderConfiguration) Serialize(writer i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.SerializationWriter)(error) {
    err := m.FraudProtectionConfiguration.Serialize(writer)
    if err != nil {
        return err
    }
    {
        err = writer.WriteObjectValue("fraudProtectionProvider", m.GetFraudProtectionProvider())
        if err != nil {
            return err
        }
    }
    return nil
}
// SetFraudProtectionProvider sets the fraudProtectionProvider property value. The fraudProtectionProvider property
func (m *FraudProtectionProviderConfiguration) SetFraudProtectionProvider(value FraudProtectionProviderable)() {
    err := m.GetBackingStore().Set("fraudProtectionProvider", value)
    if err != nil {
        panic(err)
    }
}
type FraudProtectionProviderConfigurationable interface {
    FraudProtectionConfigurationable
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
    GetFraudProtectionProvider()(FraudProtectionProviderable)
    SetFraudProtectionProvider(value FraudProtectionProviderable)()
}
