// Code generated by go-swagger; DO NOT EDIT.

package models

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

import (
	"context"

	"github.com/go-openapi/strfmt"
	"github.com/go-openapi/swag"
)

// HashicorpCloudVaultLink20221107RaftQuorumStatus hashicorp cloud vault link 20221107 raft quorum status
//
// swagger:model hashicorp.cloud.vault_link_20221107.RaftQuorumStatus
type HashicorpCloudVaultLink20221107RaftQuorumStatus struct {

	// is healthy
	IsHealthy bool `json:"is_healthy,omitempty"`

	// quorum number
	QuorumNumber int32 `json:"quorum_number,omitempty"`

	// warning
	Warning string `json:"warning,omitempty"`
}

// Validate validates this hashicorp cloud vault link 20221107 raft quorum status
func (m *HashicorpCloudVaultLink20221107RaftQuorumStatus) Validate(formats strfmt.Registry) error {
	return nil
}

// ContextValidate validates this hashicorp cloud vault link 20221107 raft quorum status based on context it is used
func (m *HashicorpCloudVaultLink20221107RaftQuorumStatus) ContextValidate(ctx context.Context, formats strfmt.Registry) error {
	return nil
}

// MarshalBinary interface implementation
func (m *HashicorpCloudVaultLink20221107RaftQuorumStatus) MarshalBinary() ([]byte, error) {
	if m == nil {
		return nil, nil
	}
	return swag.WriteJSON(m)
}

// UnmarshalBinary interface implementation
func (m *HashicorpCloudVaultLink20221107RaftQuorumStatus) UnmarshalBinary(b []byte) error {
	var res HashicorpCloudVaultLink20221107RaftQuorumStatus
	if err := swag.ReadJSON(b, &res); err != nil {
		return err
	}
	*m = res
	return nil
}
