// Code generated by Microsoft Kiota - DO NOT EDIT.
// Changes may cause incorrect behavior and will be lost if the code is regenerated.

package models

import (
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91 "github.com/microsoft/kiota-abstractions-go/serialization"
)

type ProcessingError struct {
    ClassificationError
}
// NewProcessingError instantiates a new ProcessingError and sets the default values.
func NewProcessingError()(*ProcessingError) {
    m := &ProcessingError{
        ClassificationError: *NewClassificationError(),
    }
    return m
}
// CreateProcessingErrorFromDiscriminatorValue creates a new instance of the appropriate class based on discriminator value
// returns a Parsable when successful
func CreateProcessingErrorFromDiscriminatorValue(parseNode i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable, error) {
    return NewProcessingError(), nil
}
// GetErrorType gets the errorType property value. The errorType property
// returns a *ContentProcessingErrorType when successful
func (m *ProcessingError) GetErrorType()(*ContentProcessingErrorType) {
    val, err := m.GetBackingStore().Get("errorType")
    if err != nil {
        panic(err)
    }
    if val != nil {
        return val.(*ContentProcessingErrorType)
    }
    return nil
}
// GetFieldDeserializers the deserialization information for the current model
// returns a map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error) when successful
func (m *ProcessingError) GetFieldDeserializers()(map[string]func(i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode)(error)) {
    res := m.ClassificationError.GetFieldDeserializers()
    res["errorType"] = func (n i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.ParseNode) error {
        val, err := n.GetEnumValue(ParseContentProcessingErrorType)
        if err != nil {
            return err
        }
        if val != nil {
            m.SetErrorType(val.(*ContentProcessingErrorType))
        }
        return nil
    }
    return res
}
// Serialize serializes information the current object
func (m *ProcessingError) Serialize(writer i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.SerializationWriter)(error) {
    err := m.ClassificationError.Serialize(writer)
    if err != nil {
        return err
    }
    if m.GetErrorType() != nil {
        cast := (*m.GetErrorType()).String()
        err = writer.WriteStringValue("errorType", &cast)
        if err != nil {
            return err
        }
    }
    return nil
}
// SetErrorType sets the errorType property value. The errorType property
func (m *ProcessingError) SetErrorType(value *ContentProcessingErrorType)() {
    err := m.GetBackingStore().Set("errorType", value)
    if err != nil {
        panic(err)
    }
}
type ProcessingErrorable interface {
    ClassificationErrorable
    i878a80d2330e89d26896388a3f487eef27b0a0e6c010c493bf80be1452208f91.Parsable
    GetErrorType()(*ContentProcessingErrorType)
    SetErrorType(value *ContentProcessingErrorType)()
}
