// Code generated by smithy-go-codegen DO NOT EDIT.

package ecs

import (
	"context"
	"fmt"
	awsmiddleware "github.com/aws/aws-sdk-go-v2/aws/middleware"
	"github.com/aws/aws-sdk-go-v2/service/ecs/types"
	"github.com/aws/smithy-go/middleware"
	smithyhttp "github.com/aws/smithy-go/transport/http"
	"time"
)

// Deletes the specified daemon. The daemon must be in an ACTIVE state to be
// deleted. Deleting a daemon stops all running daemon tasks on the associated
// container instances. Amazon ECS drains existing container instances and
// provisions new instances without the deleted daemon. Amazon ECS automatically
// launches replacement tasks for your Amazon ECS services.
//
// ECS Managed Daemons is only supported for Amazon ECS Managed Instances Capacity
// Providers.
func (c *Client) DeleteDaemon(ctx context.Context, params *DeleteDaemonInput, optFns ...func(*Options)) (*DeleteDaemonOutput, error) {
	if params == nil {
		params = &DeleteDaemonInput{}
	}

	result, metadata, err := c.invokeOperation(ctx, "DeleteDaemon", params, optFns, c.addOperationDeleteDaemonMiddlewares)
	if err != nil {
		return nil, err
	}

	out := result.(*DeleteDaemonOutput)
	out.ResultMetadata = metadata
	return out, nil
}

type DeleteDaemonInput struct {

	// The Amazon Resource Name (ARN) of the daemon to delete.
	//
	// This member is required.
	DaemonArn *string

	noSmithyDocumentSerde
}

type DeleteDaemonOutput struct {

	// The Unix timestamp for the time when the daemon was created.
	CreatedAt *time.Time

	// The Amazon Resource Name (ARN) of the daemon.
	DaemonArn *string

	// The Amazon Resource Name (ARN) of the daemon deployment that was triggered by
	// the delete operation. This deployment drains existing daemon tasks from the
	// container instances.
	DeploymentArn *string

	// The status of the daemon. After you call DeleteDaemon , the status changes to
	// DELETE_IN_PROGRESS .
	Status types.DaemonStatus

	// The Unix timestamp for the time when the daemon was last updated.
	UpdatedAt *time.Time

	// Metadata pertaining to the operation's result.
	ResultMetadata middleware.Metadata

	noSmithyDocumentSerde
}

func (c *Client) addOperationDeleteDaemonMiddlewares(stack *middleware.Stack, options Options) (err error) {
	if err := stack.Serialize.Add(&setOperationInputMiddleware{}, middleware.After); err != nil {
		return err
	}
	err = stack.Serialize.Add(&awsAwsjson11_serializeOpDeleteDaemon{}, middleware.After)
	if err != nil {
		return err
	}
	err = stack.Deserialize.Add(&awsAwsjson11_deserializeOpDeleteDaemon{}, middleware.After)
	if err != nil {
		return err
	}
	if err := addProtocolFinalizerMiddlewares(stack, options, "DeleteDaemon"); err != nil {
		return fmt.Errorf("add protocol finalizers: %v", err)
	}

	if err = addlegacyEndpointContextSetter(stack, options); err != nil {
		return err
	}
	if err = addSetLoggerMiddleware(stack, options); err != nil {
		return err
	}
	if err = addClientRequestID(stack); err != nil {
		return err
	}
	if err = addComputeContentLength(stack); err != nil {
		return err
	}
	if err = addResolveEndpointMiddleware(stack, options); err != nil {
		return err
	}
	if err = addComputePayloadSHA256(stack); err != nil {
		return err
	}
	if err = addRetry(stack, options, c); err != nil {
		return err
	}
	if err = addRawResponseToMetadata(stack); err != nil {
		return err
	}
	if err = addRecordResponseTiming(stack); err != nil {
		return err
	}
	if err = addSpanRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addClientUserAgent(stack, options); err != nil {
		return err
	}
	if err = smithyhttp.AddErrorCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = smithyhttp.AddCloseResponseBodyMiddleware(stack); err != nil {
		return err
	}
	if err = addSetLegacyContextSigningOptionsMiddleware(stack); err != nil {
		return err
	}
	if err = addUserAgentRetryMode(stack, options); err != nil {
		return err
	}
	if err = addCredentialSource(stack, options); err != nil {
		return err
	}
	if err = addOpDeleteDaemonValidationMiddleware(stack); err != nil {
		return err
	}
	if err = stack.Initialize.Add(newServiceMetadataMiddleware_opDeleteDaemon(options.Region), middleware.Before); err != nil {
		return err
	}
	if err = addRecursionDetection(stack); err != nil {
		return err
	}
	if err = addRequestIDRetrieverMiddleware(stack); err != nil {
		return err
	}
	if err = addResponseErrorMiddleware(stack); err != nil {
		return err
	}
	if err = addRequestResponseLogging(stack, options); err != nil {
		return err
	}
	if err = addDisableHTTPSMiddleware(stack, options); err != nil {
		return err
	}
	if err = addInterceptBeforeRetryLoop(stack, options); err != nil {
		return err
	}
	if err = addInterceptAttempt(stack, options); err != nil {
		return err
	}
	if err = addInterceptors(stack, options); err != nil {
		return err
	}
	return nil
}

func newServiceMetadataMiddleware_opDeleteDaemon(region string) *awsmiddleware.RegisterServiceMetadata {
	return &awsmiddleware.RegisterServiceMetadata{
		Region:        region,
		ServiceID:     ServiceID,
		OperationName: "DeleteDaemon",
	}
}
