// Code generated by "stringer -type=RoundMode"; DO NOT EDIT.

//go:build go1.18

package kernels

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[RoundDown-0]
	_ = x[RoundUp-1]
	_ = x[TowardsZero-2]
	_ = x[AwayFromZero-3]
	_ = x[HalfDown-4]
	_ = x[HalfUp-5]
	_ = x[HalfTowardsZero-6]
	_ = x[HalfAwayFromZero-7]
	_ = x[HalfToEven-8]
	_ = x[HalfToOdd-9]
}

const _RoundMode_name = "RoundDownRoundUpTowardsZeroAwayFromZeroHalfDownHalfUpHalfTowardsZeroHalfAwayFromZeroHalfToEvenHalfToOdd"

var _RoundMode_index = [...]uint8{0, 9, 16, 27, 39, 47, 53, 68, 84, 94, 103}

func (i RoundMode) String() string {
	if i < 0 || i >= RoundMode(len(_RoundMode_index)-1) {
		return "RoundMode(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _RoundMode_name[_RoundMode_index[i]:_RoundMode_index[i+1]]
}
